--*******************************************************************************
-- Actel Corportation
-- Fusion's Voltage Comparator Application Note's Design Example
--
-- File Name:    TOP_CM.vhd
-- Date:         December 2006
-- Language:     VHDL
--
--********************************************************************************
-- IMPORTANT-READ THESE TERMS CAREFULLY BEFORE UTILIZING THE INFORMATION
-- CONTAINED IN THIS APPLICATION NOTE. 
--
-- NO SUPPORT/NO WARRANTY. THE DESIGN FILES AND DOCUMENTATION ARE PROVIDED
-- WITHOUT SUPPORT OR WARRANTY OF ANY KIND. ACTEL IS NOT OBLIGATED TO 
-- PROVIDE UPDATES, BUG FIXES, OR TECHNICAL SUPPORT AND DISCLAIMS ALL 
-- WARRANTIES INCLUDING, WITHOUT LIMITATION, THE WARRANTIES OF MERCHANTABILITY,
-- FITNESS FOR A PARTICULAR PURPOSE, AND WARRANTIES OF NON-INFRINGEMENT OF THE
-- RIGHTS OF THIRD PARTIES (INCLUDING, WITHOUT LIMITATION, RIGHTS UNDER PATENT,
-- COPYRIGHT, TRADE SECRET, OR OTHER INTELLECTUAL PROPERTY RIGHTS).  RECIPIENT
-- ACCEPTS THE APPLICATION NOTE DESIGN FILES AND DOCUMENTATION IN "AS-IS"
-- CONDITION.
--*******************************************************************************

library ieee;
use ieee.std_logic_1164.all;
library fusion;

-- This entity uses internal voltage regulator of Fusion devices (PUB and SPARE ports are related
-- to the voltage regulator. The usage is optional. Users can use externally supplied 1.5V to power up
-- the FPGA core.

entity TOP_CM is 
generic (WIDTH_OUTPUT : integer := 12);
port(
SYSCLK : in std_logic; -- 10MHz system clock input
SYSRSTn : in std_logic; -- active low system reset
VAREF : inout std_logic;
AV0 : in std_logic; -- P port input for Diff measurement channel 0
AC0 : in std_logic; -- N port input for Diff measurement channel 0 
AV1 : in std_logic; -- P port input for Diff measurement channel 1
AC1 : in std_logic; -- N port input for Diff measurement channel 1
AV2 : in std_logic; -- P port input for Diff measurement channel 2
AC2 : in std_logic; -- N port input for Diff measurement channel 2
AV3 : in std_logic; -- P port input for Diff measurement channel 3
AC3 : in std_logic; -- N port input for Diff measurement channel 3
AV4 : in std_logic; -- P port input for Diff measurement channel 4
AC4 : in std_logic; -- N port input for Diff measurement channel 4
AV5 : in std_logic; -- P port input for Diff measurement channel 5
AC5 : in std_logic; -- N port input for Diff measurement channel 5 
Diff0: out std_logic_vector(WIDTH_OUTPUT-1 downto 0); -- differential measurement results
Diff1: out std_logic_vector(WIDTH_OUTPUT-1 downto 0); -- differential measurement results
Diff2: out std_logic_vector(WIDTH_OUTPUT-1 downto 0); -- differential measurement results
Diff3: out std_logic_vector(WIDTH_OUTPUT-1 downto 0); -- differential measurement results
Diff4: out std_logic_vector(WIDTH_OUTPUT-1 downto 0); -- differential measurement results
Diff5: out std_logic_vector(WIDTH_OUTPUT-1 downto 0); -- differential measurement results
Latest_Update: out std_logic_vector (2 downto 0); -- index of latest measured channel
New_Measurement: out std_logic; -- output flagging arrival of a new measurement result. active for one clock cycle
PUB: in std_logic;
SPARE: out std_logic
);
end TOP_CM;

architecture Comparator of TOP_CM is 

--Analog Block
component AB_CM 
port (SYS_CLK, SYS_RESET : in std_logic; VAREF : inout
         std_logic := 'Z'; DATAVALID : out std_logic; AV0, AV1, 
        AV2, AV3, AV4, AV5, AC0, AC1, AC2, AC3, AC4, AC5 : in
         std_logic; ASSC_DONE, ASSC_WAIT, ASSC_CHSAT, ASSC_CHLATD : 
        out std_logic; INIT_ADDR : 
        in std_logic_vector(8 downto 0); INIT_DATA : 
        in std_logic_vector(8 downto 0); INIT_ACM_WEN, 
        INIT_ASSC_WEN, INIT_EV_WEN, INIT_TR_WEN, INIT_DONE : in
         std_logic; ADC_RESULT : 
        out std_logic_vector(11 downto 0); ADC_CHNUMBER : 
        out std_logic_vector(4 downto 0));
end component;

-- Flash memory
component NVM_CM 
    port(INIT_CLK, SYS_RESET, INIT_POWER_UP : in std_logic; 
        INIT_DONE : out std_logic; INIT_DATA : 
        out std_logic_vector(8 downto 0); INIT_ADDR : 
        out std_logic_vector(8 downto 0); INIT_ACM_WEN, 
        INIT_ASSC_WEN, INIT_EV_WEN, INIT_TR_WEN : out std_logic);
end component;

-- internal 1.5 VR. Usage is optional
component VR_INT 
    port(PUB, VRPU : in std_logic;  FPGAGOOD, PUCORE : out 
        std_logic) ;
end component;

-- State machine perofrming all the Diff measurements 
component COMP_SM  is 
 generic (WIDTH : integer := 12);
 port
 (SYSCLK: in std_logic;
  RESETn: in std_logic;
  DATAVALID: in std_logic;
  INIT_DONE: in std_logic;
  ADC_CHNUMBER: in std_logic_vector(4 downto 0);
  ADC_RESULT: in std_logic_vector(WIDTH_OUTPUT-1 downto 0);
  Diff_0,Diff_1, Diff_2, Diff_3, Diff_4, Diff_5: out std_logic_vector(WIDTH_OUTPUT-1 downto 0);
  Latest_Update: out std_logic_vector(2 downto 0);
  New_Measurement: out std_logic
  );
  end component;

signal ASSC_DONE, DATAVALID : std_logic; 
signal ADC_RESULT : std_logic_vector(WIDTH_OUTPUT-1 downto 0); 
signal ADC_CHNUMBER: std_logic_vector(4 downto 0); 
signal INIT_ADDR, INIT_DATA: std_logic_vector (8 downto 0);
signal INIT_ACM_WEN, INIT_DONE, INIT_ASSC_WEN, INIT_EV_WEN, INIT_TR_WEN: std_logic;
signal FPGAGOOD: std_logic;

begin

AB_CM_inst: AB_CM port map 
 (SYS_CLK => SYSCLK,
  SYS_RESET => SYSRSTn,
  VAREF => VAREF,
  DATAVALID => DATAVALID,
  AV0 => AV0,
  AV1 => AV1,
  AV2 => AV2,
  AV3 => AV3,
  AV4 => AV4,
  AV5 => AV5,
  AC0 => AC0,
  AC1 => AC1,
  AC2 => AC2,
  AC3 => AC3,
  AC4 => AC4,
  AC5 => AC5,
  ASSC_DONE => ASSC_DONE,
  ASSC_WAIT => OPEN,
  ASSC_CHSAT => OPEN,
  ASSC_CHLATD => OPEN,
  INIT_ADDR => INIT_ADDR,
  INIT_DATA => INIT_DATA,
  INIT_ACM_WEN => INIT_ACM_WEN,
  INIT_ASSC_WEN => INIT_ASSC_WEN,
  INIT_EV_WEN => INIT_EV_WEN,
  INIT_TR_WEN => INIT_TR_WEN,
  INIT_DONE => INIT_DONE,
  ADC_RESULT => ADC_RESULT,
  ADC_CHNUMBER => ADC_CHNUMBER
  );

NVM_CM_inst: NVM_CM port map 
    (INIT_CLK => SYSCLK,
     SYS_RESET => SYSRSTn,
     INIT_POWER_UP => '1',
     INIT_DONE => INIT_DONE,
     INIT_DATA => INIT_DATA,
     INIT_ADDR => INIT_ADDR,
     INIT_ACM_WEN => INIT_ACM_WEN,
     INIT_ASSC_WEN => INIT_ASSC_WEN,
     INIT_EV_WEN => INIT_EV_WEN,
     INIT_TR_WEN => INIT_TR_WEN
     );

VR_INT_inst: VR_INT port map 
    (PUB => PUB,
     VRPU => '0',
     FPGAGOOD => FPGAGOOD,
     PUCORE => OPEN
     ) ;
SPARE <= FPGAGOOD;

COMP_SM_inst: COMP_SM
 generic map (WIDTH_OUTPUT)
 port map 
 (SYSCLK => SYSCLK,
  RESETn => SYSRSTn,
  -- In this design, state machine is using ASSC_DONE as indication of ADC conversion completion
  DATAVALID => ASSC_DONE,
  ADC_CHNUMBER => ADC_CHNUMBER,
  ADC_RESULT => ADC_RESULT,
  INIT_DONE => INIT_DONE,
  Diff_0 => Diff0,
  Diff_1 => Diff1,
  Diff_2 => Diff2,
  Diff_3 => Diff3,
  Diff_4 => Diff4,
  Diff_5 => Diff5,
  Latest_Update => Latest_Update,
  New_Measurement => New_Measurement
  );

end Comparator;

